// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//	xmmexposure.h --- abstract class for updating exposure information
//
//	DESCRIPTION:
//
//	AUTHOR:
//		D. Hinshaw
//

#ifndef XMMEXPOSE
#define XMMEXPOSE

#include "Dal.h"
#include "Gti.h"
#include "Xmm.h"
#include "Xmm.h"
#include <numeric>

class XmmExposure {

public:

  XmmExposure(Table *eventTable);

  void getExposureInformation();

  void updateOntime();
  
  void updateLivetime();

  void writeDSlineKeys();

  real64 liveTime(unsigned ccdNr);

  string convSpecialExpression(string &e, string &fname);

private:

  struct AttString { std::string::size_type pos; unsigned len; };
  void _getGtiLists();
  real64 _getLiveTime(unsigned ccdRef);
  void getDSlines(unsigned iccd, vector<int> &dslvec);
  vector<AttString> findSpecialStrings(std::string &s);
  vector<size_t> findAllOccurances(std::string &s, 
                                        std::string &sub);
  string interpretImageExpr(string &att, string &fname);
  string convExpressionCeltoSky(string &att, string &fname);
  vector<double> extractVals(string &att);

  DataSet *_set;
  Table *_eventTable;
  std::vector<unsigned> _ccdNums;
  std::vector<GtiList> _gtiLists;
  int _targetCCD;
  Xmm::Instrument _instrument; 
};

#endif
